/*
 * Connection.h
 *
 *  Created on: 28.11.2009
 *      Author: wglas
 */

#ifndef FANCYDBCONNECTION_H_
#define FANCYDBCONNECTION_H_

#include <fancydb.h>
#include <fancypoolableconnection.h>

namespace Fancy {

namespace DB {

class Driver;
class Statement;
class PreparedStatement;
class Transaction;

class FANCYDB_API Connection: public virtual Fancy::PoolableConnection {
public:
	typedef Fancy::ObjPtr<Connection> ptr_t;

	/**
	 * Start a transaction in this connection. This method will throw an exception,
	 * if there is another transaction active on this connection.
	 */
	virtual Fancy::ObjPtr<Transaction> startTransaction() = 0;

	/**
	 * Create a simple SQL statement to execute.
	 */
	virtual Fancy::ObjPtr<Statement> createStatement() = 0;

	/**
	 * Create a simple SQL statement to execute and set the sql string.
	 */
	virtual Fancy::ObjPtr<Statement> createStatement(const std::string &sql) = 0;

	/**
	 * Create a Prepared Statement to execute.
	 */
	virtual Fancy::ObjPtr<PreparedStatement> createPreparedStatement() = 0;

	/**
	 * Create a Prepared Statement to execute and set the sql string.
	 */
	virtual Fancy::ObjPtr<PreparedStatement> createPreparedStatement(const std::string &sql) = 0;

	/**
	 * @return The underlying driver instance.
	 */
	virtual Fancy::ObjPtr<Driver> getDriver() const = 0;

	/**
	 * Quote a string for insertion into a simple SQL statement.
	 *
	 * @param s A string to append the quoted string to.
	 * @param data The pointer to the content to be quoted.
	 * @param data_len The length of the dat to be quoted.
	 */
	virtual void quoteString(std::string &s, const char *data, size_t data_len) const = 0;

	/**
	 * @return The default format for @c UtcTimestamp.format() used to bind
	 *         placeholders.
	 */
	virtual const char *getDefaultTimestampFormat() const = 0;

protected:
	Connection();
	virtual ~Connection();
};

}

}

#endif /* CONNECTION_H_ */
