/*
 * fancydbresult.h
 *
 *  Created on: 28.11.2009
 *      Author: wglas
 */

#ifndef FANCYDBRESULT_H_
#define FANCYDBRESULT_H_

#include <fancydb.h>
#include <fancyobject.h>

namespace Fancy {

class Integer;
class Long;
class Double;
class String;
class UtcTimestamp;
template<class T> class ArrayObject;

namespace DB {

class FANCYDB_API Result: public virtual Fancy::Object {
public:
	typedef Fancy::ObjPtr<Result> ptr_t;

	/**
	 * Switch to the next row of the result set.
	 * @return Whether a next row is available or the last record
	 *         in this result set has already been reached.
	 */
	virtual bool nextRow() = 0;

	/**
	 * @return The number of columns in the result set.
	 */
	virtual size_t getColumnCount() const = 0;

	/**
	 * @param column column number, 0 .. getColumnCount()-1
	 */
	virtual bool                               isNull(size_t column) const = 0;

	/**
	 * @param column column number, 0 .. getColumnCount()-1
	 */
	virtual int32_t                            getInt(size_t column) const = 0;

	/**
	 * @param column column number, 0 .. getColumnCount()-1
	 */
	virtual Fancy::ObjPtr<Fancy::Integer>      getIntObj(size_t column) const = 0;

	/**
	 * @param column column number, 0 .. getColumnCount()-1
	 */
	virtual int64_t                            getLong(size_t column) const = 0;

	/**
	 * @param column column number, 0 .. getColumnCount()-1
	 */
	virtual Fancy::ObjPtr<Fancy::Long>         getLongObj(size_t column) const = 0;

	/**
	 * @param column column number, 0 .. getColumnCount()-1
	 */
	virtual double                             getDouble(size_t column) const = 0;

	/**
	 * @param column column number, 0 .. getColumnCount()-1
	 */
	virtual Fancy::ObjPtr<Fancy::Double>       getDoubleObj(size_t column) const = 0;

	/**
	 * @param column column number, 0 .. getColumnCount()-1
	 */
	virtual Fancy::ObjPtr<Fancy::String>       getString(size_t column) const = 0;

	/**
	 * @param column column number, 0 .. getColumnCount()-1
	 */
	virtual std::string                        getStdString(size_t column) const = 0;

	/**
	 * @param column column number, 0 .. getColumnCount()-1
	 */
	virtual Fancy::ObjPtr<Fancy::UtcTimestamp> getTimestamp(size_t column) const = 0;

	/**
	 * @param column column number, 0 .. getColumnCount()-1
	 */
	virtual Fancy::ObjPtr<Fancy::ArrayObject<int8_t> > getBlob(size_t column) const = 0;

	/**
	 * @param column column number, 0 .. getColumnCount()-1
	 * @return The labl of the given column in the result set.
	 */
	virtual std::string getColumnLabel(size_t column) const = 0;

	/**
	 * @param lbl The label of the column to retrieve.
	 */
	virtual bool                               isNull(const std::string &lbl) const = 0;

	/**
	 * @param lbl The label of the column to retrieve.
	 */
	virtual int32_t                            getInt(const std::string &lbl) const = 0;

	/**
	 * @param lbl The label of the column to retrieve.
	 */
	virtual Fancy::ObjPtr<Fancy::Integer>      getIntObj(const std::string &lbl) const = 0;

	/**
	 * @param lbl The label of the column to retrieve.
	 */
	virtual int64_t                            getLong(const std::string &lbl) const = 0;

	/**
	 * @param lbl The label of the column to retrieve.
	 */
	virtual Fancy::ObjPtr<Fancy::Long>         getLongObj(const std::string &lbl) const = 0;

	/**
	 * @param lbl The label of the column to retrieve.
	 */
	virtual double                             getDouble(const std::string &lbl) const = 0;

	/**
	 * @param lbl The label of the column to retrieve.
	 */
	virtual Fancy::ObjPtr<Fancy::Double>       getDoubleObj(const std::string &lbl) const = 0;

	/**
	 * @param lbl The label of the column to retrieve.
	 */
	virtual Fancy::ObjPtr<Fancy::String>       getString(const std::string &lbl) const = 0;

	/**
	 * @param lbl The label of the column to retrieve.
	 */
	virtual std::string                        getStdString(const std::string &lbl) const = 0;

	/**
	 * @param lbl The label of the column to retrieve.
	 */
	virtual Fancy::ObjPtr<Fancy::UtcTimestamp> getTimestamp(const std::string &lbl) const = 0;

	/**
	 * @param lbl The label of the column to retrieve.
	 */
	virtual Fancy::ObjPtr<Fancy::ArrayObject<int8_t> > getBlob(const std::string &lbl) const = 0;

protected:
	Result();
	virtual ~Result();
};

}

}

#endif /* FANCYDBRESULT_H_ */
