/*
 * Statement.h
 *
 *  Created on: 28.11.2009
 *      Author: wglas
 */

#ifndef FANCYDBSTATEMENT_H_
#define FANCYDBSTATEMENT_H_

#include <fancydbconnectionbound.h>

namespace Fancy {

namespace DB {

class Result;

/**
 * An ordinary SQL statement.
 */
class FANCYDB_API Statement: public virtual ConnectionBound {
public:
	typedef Fancy::ObjPtr<Statement> ptr_t;

	/**
	 * Set the SQL statement to be executed. If this is a
	 * @c PreparedStatement, the SQL statement may contain
	 * question marks as placeholders.
	 */
	virtual void setSQL(const std::string &s) = 0;

	/**
	 * Execute the statement as an update or insert, which require
	 * no result set.
	 */
	virtual void executeUpdate() = 0;

	/**
	 * Execute the statement as a query, which returns a result set.
	 * @return The result set or @c null, if the SQL statement was not
	 *         a query.
	 */
	virtual Fancy::ObjPtr<Result> executeQuery() = 0;

	/**
	 * Register an automatically generated INT/BIGINT column whose generated value shall be retrieved using @getGeneratedIdValue() later.
	 * Most drivers will not support more than one such column an throw an exception if a 2nd such column is registered.
	 */
	virtual void addGeneratedIdColumn(const std::string &id) = 0;

	/**
	 * Returns the automatically generated value of a column previously registered using @addGeneratedIdColumn().
	 * @return The value of the column just generated by an executeUpdate("INSERT ...") call. 0 in case of any problems.
	 */
	virtual uint64_t getGeneratedIdValue(const std::string &id) = 0;

protected:
	Statement();
	virtual ~Statement();
};

}

}

#endif /* FANCYDBSTATEMENT_H_ */
