/*
 * fancydbtransaction.h
 *
 *  Created on: 28.11.2009
 *      Author: wglas
 */

#ifndef FANCYDBTRANSACTION_H_
#define FANCYDBTRANSACTION_H_

#include <fancydbconnectionbound.h>

namespace Fancy {

namespace DB {

class Connection;

/**
 * A database transaction as returned by @c Connection::startTransation(bool).
 */
class FANCYDB_API Transaction: public virtual ConnectionBound {
public:
	typedef Fancy::ObjPtr<Transaction> ptr_t;

	/**
	 * Commit the transaction. After calling this function, this
	 * object is unusable.
	 */
	virtual void commit() = 0;

	/**
	 * Roll back the transaction. After calling this function, this
	 * object is unusable. The transaction is automagically rolled back,
	 * if the transaction object is destroyed.
	 */
	virtual void rollback() = 0;

	/**
	 * @return Whether this transaction is still active, i.e. neither
	 * @c commit() nor @c rollback() has been called.
	 */
	virtual bool isActive() const = 0;

protected:
	Transaction();

	/**
	 * Any reimplementations of this destructor are required to
	 * call @c rollback(), if the transaction is still active.
	 * This task can not be undertaken by the Transaction destructor itself,
	 * because it is called after the implementation-specific destructor
	 * bits are called and will therefore most likely produce a crash.
	 */
	~Transaction();
};

}

}

#endif /* FANCYDBTRANSACTION_H_ */
